# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class V1MachineHistoryResponseVibrations(UniversalBaseModel):
    x: typing_extensions.Annotated[typing.Optional[float], FieldMetadata(alias="X"), pydantic.Field(alias="X")] = None
    y: typing_extensions.Annotated[typing.Optional[float], FieldMetadata(alias="Y"), pydantic.Field(alias="Y")] = None
    z: typing_extensions.Annotated[typing.Optional[float], FieldMetadata(alias="Z"), pydantic.Field(alias="Z")] = None
    time: typing.Optional[int] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

ReportRunObjectResponseBodyStatus = typing.Union[
    typing.Literal["complete", "pending", "failed", "canceled", "unknown"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class LocationDataPointGpsLocationPlace(UniversalBaseModel):
    """
    Address of the location
    """

    city: typing.Optional[str] = pydantic.Field(default=None)
    """
    City
    """

    house_number: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="houseNumber"),
        pydantic.Field(alias="houseNumber", description="House number"),
    ] = None
    neighborhood: typing.Optional[str] = pydantic.Field(default=None)
    """
    Neighborhood
    """

    poi: typing.Optional[str] = pydantic.Field(default=None)
    """
    POI
    """

    postcode: typing.Optional[str] = pydantic.Field(default=None)
    """
    Postcode
    """

    state: typing.Optional[str] = pydantic.Field(default=None)
    """
    State
    """

    street: typing.Optional[str] = pydantic.Field(default=None)
    """
    Street
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class V1TrailerAssignmentResponse(UniversalBaseModel):
    driver_id: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="driverId"),
        pydantic.Field(alias="driverId", description="The ID of the driver associated with this trailer."),
    ] = None
    end_ms: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="endMs"),
        pydantic.Field(
            alias="endMs",
            description="The time at which the driver ended the assignment. If the assignment is current, this value will be omitted.",
        ),
    ] = None
    start_ms: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="startMs"),
        pydantic.Field(alias="startMs", description="The time at which the driver started the assignment"),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..errors.bad_gateway_error import BadGatewayError
from ..errors.gateway_timeout_error import GatewayTimeoutError
from ..errors.internal_server_error import InternalServerError
from ..errors.method_not_allowed_error import MethodNotAllowedError
from ..errors.not_found_error import NotFoundError
from ..errors.not_implemented_error import NotImplementedError
from ..errors.service_unavailable_error import ServiceUnavailableError
from ..errors.too_many_requests_error import TooManyRequestsError
from ..errors.unauthorized_error import UnauthorizedError
from ..types.speeding_intervals_get_speeding_intervals_response_body import (
    SpeedingIntervalsGetSpeedingIntervalsResponseBody,
)
from .types.get_speeding_intervals_request_query_by import GetSpeedingIntervalsRequestQueryBy


class RawSpeedingIntervalsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get_speeding_intervals(
        self,
        *,
        start_time: str,
        asset_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        end_time: typing.Optional[str] = None,
        query_by: typing.Optional[GetSpeedingIntervalsRequestQueryBy] = None,
        include_asset: typing.Optional[bool] = None,
        include_driver_id: typing.Optional[bool] = None,
        after: typing.Optional[str] = None,
        severity_levels: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[SpeedingIntervalsGetSpeedingIntervalsResponseBody]:
        """
        This endpoint will return all speeding intervals associated with all trips that have been collected for your organization based on the time parameters passed in. Only completed trips are included. Trips with no speeding intervals detected will be returned with an empty list of intervals. Results are paginated.

         <b>Rate limit:</b> 5 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Read Speeding Intervals** under the Speeding Intervals category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        start_time : str
            RFC 3339 timestamp that indicates when to begin receiving data. Value is compared against `updatedAtTime` or `tripStartTime` depending on the queryBy parameter.

        asset_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Comma-separated list of asset IDs. Include up to 50 asset IDs.

        end_time : typing.Optional[str]
            RFC 3339 timestamp which is compared against `updatedAtTime` or `tripStartTime` depending on the queryBy parameter. If not provided then the endpoint behaves as an unending feed of changes.

        query_by : typing.Optional[GetSpeedingIntervalsRequestQueryBy]
            Decide which timestamp the `startTime` and `endTime` are compared to.  Valid values: `updatedAtTime`, `tripStartTime`

        include_asset : typing.Optional[bool]
            Indicates whether or not to return expanded “asset” data

        include_driver_id : typing.Optional[bool]
            Indicates whether or not to return trip's driver id

        after : typing.Optional[str]
             If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        severity_levels : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Optional string of comma-separated severity levels to filter speeding intervals by. Valid values:  “light”, ”moderate”, ”heavy”, “severe”. This filter does not exclude trips that have no speeding intervals.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[SpeedingIntervalsGetSpeedingIntervalsResponseBody]
            OK response.
        """
        _response = self._client_wrapper.httpx_client.request(
            "speeding-intervals/stream",
            method="GET",
            params={
                "assetIds": asset_ids,
                "startTime": start_time,
                "endTime": end_time,
                "queryBy": query_by,
                "includeAsset": include_asset,
                "includeDriverId": include_driver_id,
                "after": after,
                "severityLevels": severity_levels,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SpeedingIntervalsGetSpeedingIntervalsResponseBody,
                    parse_obj_as(
                        type_=SpeedingIntervalsGetSpeedingIntervalsResponseBody,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawSpeedingIntervalsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get_speeding_intervals(
        self,
        *,
        start_time: str,
        asset_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        end_time: typing.Optional[str] = None,
        query_by: typing.Optional[GetSpeedingIntervalsRequestQueryBy] = None,
        include_asset: typing.Optional[bool] = None,
        include_driver_id: typing.Optional[bool] = None,
        after: typing.Optional[str] = None,
        severity_levels: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[SpeedingIntervalsGetSpeedingIntervalsResponseBody]:
        """
        This endpoint will return all speeding intervals associated with all trips that have been collected for your organization based on the time parameters passed in. Only completed trips are included. Trips with no speeding intervals detected will be returned with an empty list of intervals. Results are paginated.

         <b>Rate limit:</b> 5 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Read Speeding Intervals** under the Speeding Intervals category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        start_time : str
            RFC 3339 timestamp that indicates when to begin receiving data. Value is compared against `updatedAtTime` or `tripStartTime` depending on the queryBy parameter.

        asset_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Comma-separated list of asset IDs. Include up to 50 asset IDs.

        end_time : typing.Optional[str]
            RFC 3339 timestamp which is compared against `updatedAtTime` or `tripStartTime` depending on the queryBy parameter. If not provided then the endpoint behaves as an unending feed of changes.

        query_by : typing.Optional[GetSpeedingIntervalsRequestQueryBy]
            Decide which timestamp the `startTime` and `endTime` are compared to.  Valid values: `updatedAtTime`, `tripStartTime`

        include_asset : typing.Optional[bool]
            Indicates whether or not to return expanded “asset” data

        include_driver_id : typing.Optional[bool]
            Indicates whether or not to return trip's driver id

        after : typing.Optional[str]
             If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        severity_levels : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Optional string of comma-separated severity levels to filter speeding intervals by. Valid values:  “light”, ”moderate”, ”heavy”, “severe”. This filter does not exclude trips that have no speeding intervals.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[SpeedingIntervalsGetSpeedingIntervalsResponseBody]
            OK response.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "speeding-intervals/stream",
            method="GET",
            params={
                "assetIds": asset_ids,
                "startTime": start_time,
                "endTime": end_time,
                "queryBy": query_by,
                "includeAsset": include_asset,
                "includeDriverId": include_driver_id,
                "after": after,
                "severityLevels": severity_levels,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SpeedingIntervalsGetSpeedingIntervalsResponseBody,
                    parse_obj_as(
                        type_=SpeedingIntervalsGetSpeedingIntervalsResponseBody,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

WorkOrderAttachmentObjectResponseBodyProcessingStatus = typing.Union[
    typing.Literal["unknown", "processing", "finished"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class FormsSelectOptionObjectResponseBody(UniversalBaseModel):
    """
    Multiple choice or checkbox value option object.
    """

    id: str = pydantic.Field()
    """
    Identifier of the option.
    """

    ignore_question_from_score_if_selected: typing_extensions.Annotated[
        typing.Optional[bool],
        FieldMetadata(alias="ignoreQuestionFromScoreIfSelected"),
        pydantic.Field(
            alias="ignoreQuestionFromScoreIfSelected",
            description="Indicates whether the question should be ignored from the total score if this option is selected. Returns true if a score weight was not given to this option. Only present when the select form field has scoring.",
        ),
    ] = None
    label: str = pydantic.Field()
    """
    Label of the option.
    """

    option_score_weight: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="optionScoreWeight"),
        pydantic.Field(
            alias="optionScoreWeight",
            description="Score weight of the option, indicates number of score points received if this option is selected. Only present if the select form field has scoring.",
        ),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class V1CargoResponseSensors(UniversalBaseModel):
    cargo_empty: typing_extensions.Annotated[
        typing.Optional[bool],
        FieldMetadata(alias="cargoEmpty"),
        pydantic.Field(alias="cargoEmpty", description="Flag indicating whether the current cargo is empty or loaded."),
    ] = None
    cargo_status_time: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="cargoStatusTime"),
        pydantic.Field(
            alias="cargoStatusTime", description="The timestamp of reported cargo status, specified in RFC 3339 time."
        ),
    ] = None
    id: typing.Optional[int] = pydantic.Field(default=None)
    """
    ID of the sensor.
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    Name of the sensor.
    """

    red_eye_distance: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="redEyeDistance"),
        pydantic.Field(
            alias="redEyeDistance", description="The distance between red eye detector and the closest object in cm."
        ),
    ] = None
    trailer_id: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="trailerId"),
        pydantic.Field(
            alias="trailerId",
            description="ID of the trailer associated with the sensor for the data point. If no trailer is connected, this parameter will not be reported.",
        ),
    ] = None
    vehicle_id: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="vehicleId"),
        pydantic.Field(
            alias="vehicleId",
            description="ID of the vehicle associated with the sensor for the data point. If no vehicle is connected, this parameter will not be reported.",
        ),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .issue_source_object_response_body_type import IssueSourceObjectResponseBodyType


class IssueSourceObjectResponseBody(UniversalBaseModel):
    """
    Contains information about where an issue came from.
    """

    id: typing.Optional[str] = pydantic.Field(default=None)
    """
    ID of the issue's source object. The format depends on the 'type'. Included if 'type' is not 'ad-hoc'.
    """

    type: IssueSourceObjectResponseBodyType = pydantic.Field()
    """
    The type of issue source.  Valid values: `form`, `ad-hoc`
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

DriverEldRulesetShift = typing.Union[
    typing.Literal["US Interstate Property", "US Interstate Passenger", "Texas Intrastate"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .conditional_field_section_object_response_body import ConditionalFieldSectionObjectResponseBody
from .document_response_object_response_body_state import DocumentResponseObjectResponseBodyState
from .field_object_response_body import FieldObjectResponseBody
from .goa_document_type_tiny_response_response_body import GoaDocumentTypeTinyResponseResponseBody
from .goa_driver_tiny_response_response_body import GoaDriverTinyResponseResponseBody
from .goa_route_stop_tiny_response_response_body import GoaRouteStopTinyResponseResponseBody
from .goa_route_tiny_response_response_body import GoaRouteTinyResponseResponseBody
from .goa_vehicle_tiny_response_response_body import GoaVehicleTinyResponseResponseBody


class DocumentResponseObjectResponseBody(UniversalBaseModel):
    """
    A single document.
    """

    conditional_field_sections: typing_extensions.Annotated[
        typing.Optional[typing.List[ConditionalFieldSectionObjectResponseBody]],
        FieldMetadata(alias="conditionalFieldSections"),
        pydantic.Field(
            alias="conditionalFieldSections", description="List of the document conditional field sections."
        ),
    ] = None
    created_at_time: typing_extensions.Annotated[
        dt.datetime,
        FieldMetadata(alias="createdAtTime"),
        pydantic.Field(alias="createdAtTime", description="Time the document was created in RFC 3339 format."),
    ]
    document_type: typing_extensions.Annotated[
        GoaDocumentTypeTinyResponseResponseBody,
        FieldMetadata(alias="documentType"),
        pydantic.Field(alias="documentType"),
    ]
    driver: GoaDriverTinyResponseResponseBody
    fields: typing.List[FieldObjectResponseBody] = pydantic.Field()
    """
    The fields associated with this document.
    """

    id: str = pydantic.Field()
    """
    Universally unique identifier for the document.
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    Name of the document.
    """

    notes: typing.Optional[str] = pydantic.Field(default=None)
    """
    Notes on the document.
    """

    route: typing.Optional[GoaRouteTinyResponseResponseBody] = None
    route_stop: typing_extensions.Annotated[
        typing.Optional[GoaRouteStopTinyResponseResponseBody],
        FieldMetadata(alias="routeStop"),
        pydantic.Field(alias="routeStop"),
    ] = None
    state: DocumentResponseObjectResponseBodyState = pydantic.Field()
    """
    The condition of the document created for the driver. Can be either Required or Submitted. Required documents are pre-populated documents for the Driver to fill out in the Driver App and have not yet been submitted. Submitted documents have been submitted by the driver in the Driver App. Archived documents have been archived by the admin in the cloud dashboard.  Valid values: `submitted`, `required`, `archived`
    """

    updated_at_time: typing_extensions.Annotated[
        typing.Optional[dt.datetime],
        FieldMetadata(alias="updatedAtTime"),
        pydantic.Field(alias="updatedAtTime", description="Time the document was updated in RFC 3339 format."),
    ] = None
    vehicle: typing.Optional[GoaVehicleTinyResponseResponseBody] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .goa_vehicle_tiny_response_response_body import GoaVehicleTinyResponseResponseBody
from .idling_report_event_address_response_body import IdlingReportEventAddressResponseBody


class IdlingReportEventResponseBody(UniversalBaseModel):
    """
    A summary of the idling event.
    """

    address: IdlingReportEventAddressResponseBody
    duration_ms: typing_extensions.Annotated[
        int,
        FieldMetadata(alias="durationMs"),
        pydantic.Field(alias="durationMs", description="The duration of this idling event in milliseconds."),
    ]
    end_time: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="endTime"),
        pydantic.Field(alias="endTime", description="The end time of this idling event in RFC 3339 format."),
    ]
    fuel_consumption_ml: typing_extensions.Annotated[
        float,
        FieldMetadata(alias="fuelConsumptionMl"),
        pydantic.Field(
            alias="fuelConsumptionMl",
            description="The amount of fuel consumed in milliliters during this idling event.",
        ),
    ]
    is_pto_active: typing_extensions.Annotated[
        bool,
        FieldMetadata(alias="isPtoActive"),
        pydantic.Field(
            alias="isPtoActive", description="Whether or not power take-off was active during this idling event."
        ),
    ]
    start_time: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="startTime"),
        pydantic.Field(alias="startTime", description="The start time of this idling event in RFC 3339 format."),
    ]
    vehicle: GoaVehicleTinyResponseResponseBody

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .quantity_object_response_body import QuantityObjectResponseBody
from .route_stop_object_response_body import RouteStopObjectResponseBody


class RouteObjectResponseBody(UniversalBaseModel):
    """
    Route object
    """

    cost: float = pydantic.Field()
    """
    The cost of the route
    """

    created_at: typing_extensions.Annotated[
        dt.datetime,
        FieldMetadata(alias="createdAt"),
        pydantic.Field(alias="createdAt", description="The timestamp (in UTC) when the route was created"),
    ]
    dispatch_route_id: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="dispatchRouteId"),
        pydantic.Field(alias="dispatchRouteId", description="The dispatch route identifier"),
    ] = None
    distance_meters: typing_extensions.Annotated[
        int,
        FieldMetadata(alias="distanceMeters"),
        pydantic.Field(alias="distanceMeters", description="The total distance of the route in meters"),
    ]
    duration_seconds: typing_extensions.Annotated[
        int,
        FieldMetadata(alias="durationSeconds"),
        pydantic.Field(alias="durationSeconds", description="The total duration of the route in seconds"),
    ]
    hub_id: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="hubId"),
        pydantic.Field(alias="hubId", description="The ID of the hub this route belongs to"),
    ]
    id: str = pydantic.Field()
    """
    The Samsara-generated unique identifier (UUID) for the route
    """

    is_edited: typing_extensions.Annotated[
        bool,
        FieldMetadata(alias="isEdited"),
        pydantic.Field(alias="isEdited", description="Whether the route has been edited"),
    ]
    is_pinned: typing_extensions.Annotated[
        bool,
        FieldMetadata(alias="isPinned"),
        pydantic.Field(alias="isPinned", description="Whether the route is pinned"),
    ]
    name: str = pydantic.Field()
    """
    The name of the route
    """

    org_location_timezone: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="orgLocationTimezone"),
        pydantic.Field(
            alias="orgLocationTimezone", description="The organization location timezone calculated from hub"
        ),
    ]
    plan_id: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="planId"),
        pydantic.Field(alias="planId", description="The ID of the plan this route belongs to"),
    ]
    quantities: typing.Optional[typing.List[QuantityObjectResponseBody]] = pydantic.Field(default=None)
    """
    List of quantity information for the route
    """

    scheduled_route_end_time: typing_extensions.Annotated[
        dt.datetime,
        FieldMetadata(alias="scheduledRouteEndTime"),
        pydantic.Field(
            alias="scheduledRouteEndTime", description="The scheduled end time of the route calculated from last stop"
        ),
    ]
    scheduled_route_start_time: typing_extensions.Annotated[
        dt.datetime,
        FieldMetadata(alias="scheduledRouteStartTime"),
        pydantic.Field(
            alias="scheduledRouteStartTime",
            description="The scheduled start time of the route calculated from first stop",
        ),
    ]
    stops: typing.List[RouteStopObjectResponseBody] = pydantic.Field()
    """
    List of stops on the route
    """

    type: str = pydantic.Field()
    """
    The type of route
    """

    updated_at: typing_extensions.Annotated[
        dt.datetime,
        FieldMetadata(alias="updatedAt"),
        pydantic.Field(alias="updatedAt", description="The timestamp (in UTC) when the route was last updated"),
    ]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .fuel_volume_response_body import FuelVolumeResponseBody
from .vehicle_with_gateway_tiny_response_response_body import VehicleWithGatewayTinyResponseResponseBody
from .workflow_address_event_with_geofence_object_response_body import (
    WorkflowAddressEventWithGeofenceObjectResponseBody,
)


class WorkflowGeofenceEventResponseObjectResponseBody(UniversalBaseModel):
    address: typing.Optional[WorkflowAddressEventWithGeofenceObjectResponseBody] = None
    fuel_volume: typing_extensions.Annotated[
        typing.Optional[FuelVolumeResponseBody], FieldMetadata(alias="fuelVolume"), pydantic.Field(alias="fuelVolume")
    ] = None
    vehicle: typing.Optional[VehicleWithGatewayTinyResponseResponseBody] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class V1VisionRunsResponseReportMetadata(UniversalBaseModel):
    """
    The response includes 4 additional fields that are now deprecated
    """

    items_per_minute: typing_extensions.Annotated[
        typing.Optional[float],
        FieldMetadata(alias="itemsPerMinute"),
        pydantic.Field(
            alias="itemsPerMinute",
            description="Returns average scanned items per minute. Should be used instead of scanRate.",
        ),
    ] = None
    no_read_count: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="noReadCount"),
        pydantic.Field(
            alias="noReadCount",
            description="Returns no read count for the run. Should be used instead of noReadScansCount",
        ),
    ] = None
    reject_count: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="rejectCount"),
        pydantic.Field(
            alias="rejectCount",
            description="Returns reject count for the run. Should be used instead of failedScansCount",
        ),
    ] = None
    success_count: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="successCount"),
        pydantic.Field(
            alias="successCount",
            description="Returns success count for the run. Should be used instead of successfulScansCount",
        ),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .alert_object_driver_response_body import AlertObjectDriverResponseBody
from .alert_object_vehicle_response_body import AlertObjectVehicleResponseBody
from .event_location_response_body import EventLocationResponseBody
from .worker_safety_sos_data_response_body_source_type import WorkerSafetySosDataResponseBodySourceType


class WorkerSafetySosDataResponseBody(UniversalBaseModel):
    """
    Details specific to Worker Safety SOS.
    """

    driver: typing.Optional[AlertObjectDriverResponseBody] = None
    location: typing.Optional[EventLocationResponseBody] = None
    source_type: typing_extensions.Annotated[
        WorkerSafetySosDataResponseBodySourceType,
        FieldMetadata(alias="sourceType"),
        pydantic.Field(
            alias="sourceType",
            description="The source of the SOS signal.  Valid values: `appSos`, `noResponseCheckIn`, `wearableSos`, `fallDetected`",
        ),
    ]
    vehicle: typing.Optional[AlertObjectVehicleResponseBody] = None
    worker_safety_incident_url: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="workerSafetyIncidentUrl"),
        pydantic.Field(
            alias="workerSafetyIncidentUrl", description="URL of the Worker Safety incident in the cloud dashboard."
        ),
    ]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .safety_event_behavior_labels_response_body import SafetyEventBehaviorLabelsResponseBody
from .safety_event_driver_object_response_body import SafetyEventDriverObjectResponseBody
from .safety_event_vehicle_object_response_body import SafetyEventVehicleObjectResponseBody


class SafetyEventObjectResponseBody(UniversalBaseModel):
    """
    The safety event that was updated.
    """

    behavior_labels: typing_extensions.Annotated[
        typing.Optional[typing.List[SafetyEventBehaviorLabelsResponseBody]],
        FieldMetadata(alias="behaviorLabels"),
        pydantic.Field(alias="behaviorLabels", description="Behavior labels for a safety event."),
    ] = None
    driver: typing.Optional[SafetyEventDriverObjectResponseBody] = None
    id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The unique Samsara ID of the safety event.
    """

    time: typing.Optional[str] = pydantic.Field(default=None)
    """
    The time the safety event occurred in RFC 3339 milliseconds format.
    """

    vehicle: typing.Optional[SafetyEventVehicleObjectResponseBody] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .alert_object_driver_response_body import AlertObjectDriverResponseBody
from .alert_object_trailer_response_body import AlertObjectTrailerResponseBody
from .alert_object_vehicle_response_body import AlertObjectVehicleResponseBody
from .speed_data_response_body_operation import SpeedDataResponseBodyOperation


class SpeedDataResponseBody(UniversalBaseModel):
    """
    Details specific to Speed.
    """

    current_speed_kilometers_per_hour: typing_extensions.Annotated[
        int,
        FieldMetadata(alias="currentSpeedKilometersPerHour"),
        pydantic.Field(
            alias="currentSpeedKilometersPerHour", description="Current speed of the vehicle in kilometers per hour."
        ),
    ]
    driver: typing.Optional[AlertObjectDriverResponseBody] = None
    min_duration_milliseconds: typing_extensions.Annotated[
        int,
        FieldMetadata(alias="minDurationMilliseconds"),
        pydantic.Field(
            alias="minDurationMilliseconds", description="Minimum duration of the current speed in milliseconds."
        ),
    ]
    operation: SpeedDataResponseBodyOperation = pydantic.Field()
    """
    Operation of the current and threshold comparison.  Valid values: `GREATER`, `INSIDE_RANGE`, `LESS`, `OUTSIDE_RANGE`
    """

    threshold_speed_kilometers_per_hour: typing_extensions.Annotated[
        int,
        FieldMetadata(alias="thresholdSpeedKilometersPerHour"),
        pydantic.Field(
            alias="thresholdSpeedKilometersPerHour",
            description="Threshold speed of the vehicle in kilometers per hour.",
        ),
    ]
    trailer: typing.Optional[AlertObjectTrailerResponseBody] = None
    vehicle: typing.Optional[AlertObjectVehicleResponseBody] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

SafetyEventsV2PatchSafetyEventsV2BatchRequestBodyContextLabelsToAddItem = typing.Union[
    typing.Literal[
        "Congested",
        "Construction",
        "Coupling/Uncoupling",
        "Cyclist/Motorcyclist",
        "Defensive",
        "Fog",
        "Improper Seat Belt",
        "Light Traffic",
        "Moderate Traffic",
        "Moderately Drowsy",
        "Mounted Phone",
        "Night",
        "Parking Lot",
        "Passenger Mobile Usage",
        "Passenger Sleeping",
        "Passengers",
        "Pedestrians",
        "Raining",
        "Sensitive Media",
        "Slightly Drowsy",
        "Snowing",
        "Snowy Road",
        "Speed Sign Verified",
        "Very Drowsy",
        "Wet Road",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class IftaDetailJobOutputResponseBody(UniversalBaseModel):
    """
    The file outputs produced by a successfully completed job.
    """

    created_at_time: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="createdAtTime"),
        pydantic.Field(alias="createdAtTime", description=" When this file was created."),
    ]
    download_url: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="downloadUrl"),
        pydantic.Field(
            alias="downloadUrl",
            description="A url to download the generated file. The contents will be gzipped. This url has an expiration and will no longer be valid after expiration.",
        ),
    ]
    download_url_expiration_time: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="downloadUrlExpirationTime"),
        pydantic.Field(
            alias="downloadUrlExpirationTime",
            description=" The expiration time of this file's download url. Requesting this job again by ID will refresh the download urls, if expired.",
        ),
    ]
    name: str = pydantic.Field()
    """
    The name of this file.
    """

    record_count: typing_extensions.Annotated[
        int,
        FieldMetadata(alias="recordCount"),
        pydantic.Field(alias="recordCount", description="The number of records in this file."),
    ]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class RollingStopDetectionAlertSettingsObjectResponseBody(UniversalBaseModel):
    """
    AI event detection settings for the rolling stop behavior. Detection is available in vehicles with compatible dash cams.
    """

    is_enabled: typing_extensions.Annotated[
        typing.Optional[bool],
        FieldMetadata(alias="isEnabled"),
        pydantic.Field(
            alias="isEnabled", description="Indicates whether AI event detection for rolling stops is turned on."
        ),
    ] = None
    speeding_threshold_mph: typing_extensions.Annotated[
        typing.Optional[float],
        FieldMetadata(alias="speedingThresholdMph"),
        pydantic.Field(alias="speedingThresholdMph", description="Alert when speed is over this many miles per hour."),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .vehicle_stats_fault_codes_value_obdii_confirmed_dtcs import VehicleStatsFaultCodesValueObdiiConfirmedDtcs
from .vehicle_stats_fault_codes_value_obdii_diagnostic_trouble_codes_ignition_type import (
    VehicleStatsFaultCodesValueObdiiDiagnosticTroubleCodesIgnitionType,
)
from .vehicle_stats_fault_codes_value_obdii_monitor_status import VehicleStatsFaultCodesValueObdiiMonitorStatus


class VehicleStatsFaultCodesValueObdiiDiagnosticTroubleCodes(UniversalBaseModel):
    """
    Diagnostic trouble code for passenger vehicles.
    """

    confirmed_dtcs: typing_extensions.Annotated[
        typing.Optional[typing.List[VehicleStatsFaultCodesValueObdiiConfirmedDtcs]],
        FieldMetadata(alias="confirmedDtcs"),
        pydantic.Field(alias="confirmedDtcs", description="Confirmed DTC codes."),
    ] = None
    ignition_type: typing_extensions.Annotated[
        typing.Optional[VehicleStatsFaultCodesValueObdiiDiagnosticTroubleCodesIgnitionType],
        FieldMetadata(alias="ignitionType"),
        pydantic.Field(
            alias="ignitionType",
            description="The ignition type of this passenger vehicle. Valid values: `spark`, `compression`.",
        ),
    ] = None
    mil_status: typing_extensions.Annotated[
        typing.Optional[bool],
        FieldMetadata(alias="milStatus"),
        pydantic.Field(alias="milStatus", description="The MIL status, indicating a check engine light."),
    ] = None
    monitor_status: typing_extensions.Annotated[
        typing.Optional[VehicleStatsFaultCodesValueObdiiMonitorStatus],
        FieldMetadata(alias="monitorStatus"),
        pydantic.Field(alias="monitorStatus"),
    ] = None
    pending_dtcs: typing_extensions.Annotated[
        typing.Optional[typing.List[VehicleStatsFaultCodesValueObdiiConfirmedDtcs]],
        FieldMetadata(alias="pendingDtcs"),
        pydantic.Field(alias="pendingDtcs", description="Pending DTC codes."),
    ] = None
    permanent_dtcs: typing_extensions.Annotated[
        typing.Optional[typing.List[VehicleStatsFaultCodesValueObdiiConfirmedDtcs]],
        FieldMetadata(alias="permanentDtcs"),
        pydantic.Field(alias="permanentDtcs", description="Permanent DTC codes."),
    ] = None
    tx_id: typing_extensions.Annotated[
        int, FieldMetadata(alias="txId"), pydantic.Field(alias="txId", description="The TX identifier.")
    ]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class CreateAttributeRequestEntities(UniversalBaseModel):
    date_values: typing_extensions.Annotated[
        typing.Optional[typing.List[dt.date]],
        FieldMetadata(alias="dateValues"),
        pydantic.Field(
            alias="dateValues",
            description="Date values that can be associated with this attribute in RFC 3339 full-date format (YYYY-MM-DD)",
        ),
    ] = None
    entity_id: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="entityId"),
        pydantic.Field(alias="entityId", description="Entity id, based on the entity type."),
    ] = None
    external_ids: typing_extensions.Annotated[
        typing.Optional[typing.Dict[str, str]],
        FieldMetadata(alias="externalIds"),
        pydantic.Field(
            alias="externalIds",
            description="The [external IDs](https://developers.samsara.com/docs/external-ids) for the given object.",
        ),
    ] = None
    number_values: typing_extensions.Annotated[
        typing.Optional[typing.List[float]],
        FieldMetadata(alias="numberValues"),
        pydantic.Field(alias="numberValues", description="Number values that can be associated with this attribute"),
    ] = None
    string_values: typing_extensions.Annotated[
        typing.Optional[typing.List[str]],
        FieldMetadata(alias="stringValues"),
        pydantic.Field(alias="stringValues", description="String values that can be associated with this attribute"),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .health_details_response_response_body import HealthDetailsResponseResponseBody
from .health_reason_response_response_body import HealthReasonResponseResponseBody
from .health_response_response_body_health_status import HealthResponseResponseBodyHealthStatus
from .health_response_response_body_primary_health_reason import HealthResponseResponseBodyPrimaryHealthReason
from .health_response_response_body_recommended_action import HealthResponseResponseBodyRecommendedAction


class HealthResponseResponseBody(UniversalBaseModel):
    """
    Health information for the device.
    """

    health_details: typing_extensions.Annotated[
        typing.Optional[HealthDetailsResponseResponseBody],
        FieldMetadata(alias="healthDetails"),
        pydantic.Field(alias="healthDetails"),
    ] = None
    health_reasons: typing_extensions.Annotated[
        typing.Optional[typing.List[HealthReasonResponseResponseBody]],
        FieldMetadata(alias="healthReasons"),
        pydantic.Field(alias="healthReasons", description="The list of active health reasons affecting this device."),
    ] = None
    health_status: typing_extensions.Annotated[
        typing.Optional[HealthResponseResponseBodyHealthStatus],
        FieldMetadata(alias="healthStatus"),
        pydantic.Field(
            alias="healthStatus",
            description="Current overall health status of the device.  Valid values: `dataPending`, `healthy`, `needsAttention`, `needsReplacement`",
        ),
    ] = None
    primary_health_reason: typing_extensions.Annotated[
        typing.Optional[HealthResponseResponseBodyPrimaryHealthReason],
        FieldMetadata(alias="primaryHealthReason"),
        pydantic.Field(
            alias="primaryHealthReason",
            description="Primary health reason affecting the device's current health status.  Valid values: `assetUnplugged`, `cameraMisaligned`, `dataPending`, `healthy`, `inwardCameraObstruction`, `irregularRecording`, `lowDeviceBattery`, `missingEldDiagnostics`, `missingVin`, `needsReplacement`, `newlyInstalledDevice`, `noGpsSignal`, `notDetected`, `outwardCameraObstruction`, `recordingTimeRequired`, `unknown`, `vgUnplugged`",
        ),
    ] = None
    primary_health_reason_start_time: typing_extensions.Annotated[
        typing.Optional[dt.datetime],
        FieldMetadata(alias="primaryHealthReasonStartTime"),
        pydantic.Field(
            alias="primaryHealthReasonStartTime",
            description="The timestamp when the primary health reason began, in RFC3339 format.",
        ),
    ] = None
    recommended_action: typing_extensions.Annotated[
        typing.Optional[HealthResponseResponseBodyRecommendedAction],
        FieldMetadata(alias="recommendedAction"),
        pydantic.Field(
            alias="recommendedAction",
            description="Recommended steps to resolve the current health reason.\n\n* `recommendedActionAgHealthy`: Gateway is healthy. No action required.\n* `recommendedActionAgLowDeviceBatteryAG45`: The gateway battery is low. Please contact support\n* `recommendedActionAgLowDeviceBatteryAG46`: The gateway battery is low. Gateway batteries require replacement.\n* `recommendedActionAgLowDeviceBatteryAG51`: Gateway batteries require replacement.\n* `recommendedActionAgLowDeviceBatteryPoweredAG`: The gateway battery is low. Power on the asset to charge the gateway.\n* `recommendedActionAgNoGpsSignal`: Gateway has no GPS signal. Verify that the gateway is securely attached to the exterior of the asset. Ensure that the device is outside and avoid obstructions such as underground structures, heavily insulated buildings, bridges, and trees.\n* `recommendedActionAgNotDetected`: The gateway has not been detected for at least 24 hours. See troubleshooting steps to reboot the gateway.\n* `recommendedActionAgUnplugged`: The asset gateway is unplugged or cut. Check the gateway connection to ensure it is plugged in and has been installed correctly\n* `recommendedActionBLEDataPending`: This device has recently been installed and more data is required to calculate health. No action required.\n* `recommendedActionBLEHealthy`: Device is healthy. No action required.\n* `recommendedActionBLELowBattery`: The device battery is low. Please contact support.\n* `recommendedActionBLENotDetected`: The device has not been detected by a gateway in the Samsara Network for at least 30 days. Verify that the device is securely attached to the asset, not in a remote area, and minimize obstructions that may disrupt detection by nearby gateways.\n* `recommendedActionCcHealthy`: Device is healthy. No action required.\n* `recommendedActionCcLowUptime`: Please ensure all cameras are physically connected to the device.\n* `recommendedActionCcMediaInputUptime`: Please ensure the camera is properly connected to the device.\n* `recommendedActionCcNotDetected`: None\n* `recommendedActionCmCameraMisaligned`: The position of the road-facing dash cam is not aligned with the horizon. Video quality and AI detection may be impacted. Reposition dash cam and Test Dash Cam Positioning.\n* `recommendedActionCmHealthy`: Dash cam recording is healthy. No action required.\n* `recommendedActionCmInactive`: None\n* `recommendedActionCmInwardCameraObstruction`: The inward-facing dash cam does not have a clear view of the vehicle cab. Video quality and AI detection may be impacted. Remove obstruction and Test Dash Cam Positioning.\n* `recommendedActionCmIrregularRecording`: The dash cam shows no evidence of recording during the expected recording time over the last 30 days. Please contact Support.\n* `recommendedActionCmLowUptime`: The dash cam is either experiencing low recording uptime, low connectivity, or both. Please follow the troubleshooting steps and contact Support if issues persist.\n* `recommendedActionCmNewlyInstalledDevice`: This dash cam has recently been installed and at least 30 hours of recording time is required to calculate health. No action required.\n* `recommendedActionCmNotDetected`: The dash cam has not been detected within the last 30 days. Check the dash cam's connection and verify your dash cam has been installed correctly.\n* `recommendedActionCmOutwardCameraObstruction`: The outward-facing dash cam does not have a clear view of the road. Video quality and AI detection may be impacted Remove obstruction and Test Dash Cam Positioning.\n* `recommendedActionCmRecordingTimeRequired`: At least 30 hours of recording time is required to determine device health. No action required.\n* `recommendedActionCmVgUnplugged`: The vehicle gateway is unplugged and the dash cam health cannot be determined. Check the vehicle gateway connection and ensure it is plugged in and has been installed correctly.\n* `recommendedActionGatewayNewlyInstalledDevice`: This gateway has recently been installed and more data is required to calculate health. No action required.\n* `recommendedActionHealthy`: Asset is healthy. No action required.\n* `recommendedActionLowBatteryVoltage`: Asset has reported low battery voltage. Please contact support.\n* `recommendedActionOemNotReporting`: Data for this asset has not been received as expected. Please contact support.\n* `recommendedActionVehicleOff`: Asset has reported an expected power-off event, and is healthy. No action required.\n* `recommendedActionVgHealthy`: Gateway is healthy. No action required.\n* `recommendedActionVgMissingEldDiagnostics`: To ensure full compliance, request exchange.\n* `recommendedActionVgMissingEldDiagnosticsExchangeSubmitted`: Exchange submitted. Please allow 12-24 hours for exchange to appear in the orders and exchanges page.\n* `recommendedActionVgMissingEldDiagnosticsHardwareExchanged`: Install exchanged hardware.\n* `recommendedActionVgMissingEldDiagnosticsNoExchangeSubmitted`: To ensure full compliance, request exchange.\n* `recommendedActionVgMissingVin`: Add the VIN to your vehicle.\n* `recommendedActionVgNeedsReplacement`: To ensure full compliance, request exchange.\n* `recommendedActionVgNotDetected`: The vehicle gateway has not been detected for at least 24 hours. Check the vehicle gateway connection to ensure it is plugged in and view additional troubleshooting steps.\n* `recommendedActionVgVgUnplugged`: The vehicle gateway is unplugged. Check the vehicle gateway connection to ensure it is plugged in and has been installed correctly.\n* `unknown`: None\n\n\n  Valid values: `recommendedActionAgHealthy`, `recommendedActionAgLowDeviceBatteryAG45`, `recommendedActionAgLowDeviceBatteryAG46`, `recommendedActionAgLowDeviceBatteryAG51`, `recommendedActionAgLowDeviceBatteryPoweredAG`, `recommendedActionAgNoGpsSignal`, `recommendedActionAgNotDetected`, `recommendedActionAgUnplugged`, `recommendedActionBLEDataPending`, `recommendedActionBLEHealthy`, `recommendedActionBLELowBattery`, `recommendedActionBLENotDetected`, `recommendedActionCcHealthy`, `recommendedActionCcLowUptime`, `recommendedActionCcMediaInputUptime`, `recommendedActionCcNotDetected`, `recommendedActionCmCameraMisaligned`, `recommendedActionCmHealthy`, `recommendedActionCmInactive`, `recommendedActionCmInwardCameraObstruction`, `recommendedActionCmIrregularRecording`, `recommendedActionCmLowUptime`, `recommendedActionCmNewlyInstalledDevice`, `recommendedActionCmNotDetected`, `recommendedActionCmOutwardCameraObstruction`, `recommendedActionCmRecordingTimeRequired`, `recommendedActionCmVgUnplugged`, `recommendedActionGatewayNewlyInstalledDevice`, `recommendedActionHealthy`, `recommendedActionLowBatteryVoltage`, `recommendedActionOemNotReporting`, `recommendedActionVehicleOff`, `recommendedActionVgHealthy`, `recommendedActionVgMissingEldDiagnostics`, `recommendedActionVgMissingEldDiagnosticsExchangeSubmitted`, `recommendedActionVgMissingEldDiagnosticsHardwareExchanged`, `recommendedActionVgMissingEldDiagnosticsNoExchangeSubmitted`, `recommendedActionVgMissingVin`, `recommendedActionVgNeedsReplacement`, `recommendedActionVgNotDetected`, `recommendedActionVgVgUnplugged`, `unknown`",
        ),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

ReportColumnsObjectResponseBodyDataType = typing.Union[
    typing.Literal["string", "float", "integer", "timestamp", "unknown"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

GetEquipmentStatsFeedRequestTypesItem = typing.Union[
    typing.Literal[
        "gatewayEngineStates",
        "obdEngineStates",
        "fuelPercents",
        "engineRpm",
        "gatewayEngineSeconds",
        "obdEngineSeconds",
        "gatewayJ1939EngineSeconds",
        "gpsOdometerMeters",
        "gps",
        "engineTotalIdleTimeMinutes",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import typing

TrainingAssignmentNearDueDateTriggerDetailsObjectRequestBodyConditionUnits = typing.Union[
    typing.Literal["DAYS", "WEEKS"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .training_assignment_near_due_date_trigger_assignment_group_object_request_body_assignment_group_type import (
    TrainingAssignmentNearDueDateTriggerAssignmentGroupObjectRequestBodyAssignmentGroupType,
)


class TrainingAssignmentNearDueDateTriggerAssignmentGroupObjectRequestBody(UniversalBaseModel):
    """
    An assignment group of a specific course or a category can be selected for an alert.
    """

    assignment_group_type: typing_extensions.Annotated[
        TrainingAssignmentNearDueDateTriggerAssignmentGroupObjectRequestBodyAssignmentGroupType,
        FieldMetadata(alias="assignmentGroupType"),
        pydantic.Field(
            alias="assignmentGroupType", description="Assignment group type.  Valid values: `CATEGORY`, `COURSE`"
        ),
    ]
    assignment_group_uuid: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="assignmentGroupUuid"),
        pydantic.Field(alias="assignmentGroupUuid", description="The unique ID of the assignment group."),
    ]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .route_settings_response_body_route_completion_condition import RouteSettingsResponseBodyRouteCompletionCondition
from .route_settings_response_body_route_starting_condition import RouteSettingsResponseBodyRouteStartingCondition
from .route_settings_response_body_sequencing_method import RouteSettingsResponseBodySequencingMethod


class RouteSettingsResponseBody(UniversalBaseModel):
    """
    An optional dictionary, only necessary to override the defaults for route start and end conditions.
    """

    route_completion_condition: typing_extensions.Annotated[
        typing.Optional[RouteSettingsResponseBodyRouteCompletionCondition],
        FieldMetadata(alias="routeCompletionCondition"),
        pydantic.Field(
            alias="routeCompletionCondition",
            description="Defaults to 'arriveLastStop' which ends the route upon arriving at the final stop. The condition 'departLastStop' \nends the route upon departing the last stop. If 'arriveLastStop' is set, then the departure time of the final stop should not be set.  Valid values: `arriveLastStop`, `departLastStop`",
        ),
    ] = None
    route_starting_condition: typing_extensions.Annotated[
        typing.Optional[RouteSettingsResponseBodyRouteStartingCondition],
        FieldMetadata(alias="routeStartingCondition"),
        pydantic.Field(
            alias="routeStartingCondition",
            description="Defaults to 'departFirstStop' which starts the route upon departing the first stop in the route.\n The condition 'arriveFirstStop' starts the route upon arriving at the first stop in the route. If 'departFirstStop' is set,\nthe arrival time of the first stop should not be set.  Valid values: `departFirstStop`, `arriveFirstStop`",
        ),
    ] = None
    sequencing_method: typing_extensions.Annotated[
        typing.Optional[RouteSettingsResponseBodySequencingMethod],
        FieldMetadata(alias="sequencingMethod"),
        pydantic.Field(
            alias="sequencingMethod",
            description="Determines how stops are sequenced on the route. 'scheduledArrivalTime' sequences stops by their scheduled arrival times (default). 'manual' allows custom sequencing via stop.sequenceNumber. 'unknown' indicates the method is not specified.  Valid values: `unknown`, `scheduledArrivalTime`, `manual`",
        ),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..errors.bad_gateway_error import BadGatewayError
from ..errors.gateway_timeout_error import GatewayTimeoutError
from ..errors.internal_server_error import InternalServerError
from ..errors.method_not_allowed_error import MethodNotAllowedError
from ..errors.not_found_error import NotFoundError
from ..errors.not_implemented_error import NotImplementedError
from ..errors.service_unavailable_error import ServiceUnavailableError
from ..errors.too_many_requests_error import TooManyRequestsError
from ..errors.unauthorized_error import UnauthorizedError
from ..types.routes_get_route_events_stream_response_body import RoutesGetRouteEventsStreamResponseBody


class RawRouteEventsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get_route_events_stream(
        self,
        *,
        after: typing.Optional[str] = None,
        start_time: typing.Optional[str] = None,
        limit: typing.Optional[int] = None,
        include_external_ids: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[RoutesGetRouteEventsStreamResponseBody]:
        """
        Subscribes to a feed of immutable, append-only updates for routes. The initial request to this feed endpoint returns a cursor, which can be used in subsequent requests to fetch events on all routes that occurred after the last retrieved event. An initial request (with empty after and startTime parameters) will return events from the past 24 hours.

         <b>Rate limit:</b> 5 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Read Routes** under the Driver Workflow category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        after : typing.Optional[str]
             If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        start_time : typing.Optional[str]
            A start time in RFC 3339 format. Defaults to 24 hours ago if not provided. Millisecond precision and timezones are supported. (Examples: 2025-07-01T19:08:25Z, 2025-07-01T19:08:25.455Z, OR 2025-07-01T14:00:12-04:00). Cannot be more than 7 days in the past.

        limit : typing.Optional[int]
            The limit for how many events will be in the response. Default and max for this value is 200 events.

        include_external_ids : typing.Optional[bool]
            Optional boolean indicating whether to return external IDs on supported entities

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[RoutesGetRouteEventsStreamResponseBody]
            OK response.
        """
        _response = self._client_wrapper.httpx_client.request(
            "route-events/stream",
            method="GET",
            params={
                "after": after,
                "startTime": start_time,
                "limit": limit,
                "includeExternalIds": include_external_ids,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    RoutesGetRouteEventsStreamResponseBody,
                    parse_obj_as(
                        type_=RoutesGetRouteEventsStreamResponseBody,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawRouteEventsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get_route_events_stream(
        self,
        *,
        after: typing.Optional[str] = None,
        start_time: typing.Optional[str] = None,
        limit: typing.Optional[int] = None,
        include_external_ids: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[RoutesGetRouteEventsStreamResponseBody]:
        """
        Subscribes to a feed of immutable, append-only updates for routes. The initial request to this feed endpoint returns a cursor, which can be used in subsequent requests to fetch events on all routes that occurred after the last retrieved event. An initial request (with empty after and startTime parameters) will return events from the past 24 hours.

         <b>Rate limit:</b> 5 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Read Routes** under the Driver Workflow category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        after : typing.Optional[str]
             If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        start_time : typing.Optional[str]
            A start time in RFC 3339 format. Defaults to 24 hours ago if not provided. Millisecond precision and timezones are supported. (Examples: 2025-07-01T19:08:25Z, 2025-07-01T19:08:25.455Z, OR 2025-07-01T14:00:12-04:00). Cannot be more than 7 days in the past.

        limit : typing.Optional[int]
            The limit for how many events will be in the response. Default and max for this value is 200 events.

        include_external_ids : typing.Optional[bool]
            Optional boolean indicating whether to return external IDs on supported entities

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[RoutesGetRouteEventsStreamResponseBody]
            OK response.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "route-events/stream",
            method="GET",
            params={
                "after": after,
                "startTime": start_time,
                "limit": limit,
                "includeExternalIds": include_external_ids,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    RoutesGetRouteEventsStreamResponseBody,
                    parse_obj_as(
                        type_=RoutesGetRouteEventsStreamResponseBody,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .route_event_details_response_body import RouteEventDetailsResponseBody
from .route_event_response_response_body_event_type import RouteEventResponseResponseBodyEventType
from .route_event_route_response_response_body import RouteEventRouteResponseResponseBody
from .route_event_stop_response_response_body import RouteEventStopResponseResponseBody


class RouteEventResponseResponseBody(UniversalBaseModel):
    """
    A single route event.
    """

    event_details: typing_extensions.Annotated[
        typing.Optional[RouteEventDetailsResponseBody],
        FieldMetadata(alias="eventDetails"),
        pydantic.Field(alias="eventDetails"),
    ] = None
    event_time: typing_extensions.Annotated[
        dt.datetime,
        FieldMetadata(alias="eventTime"),
        pydantic.Field(alias="eventTime", description="Time the event was processed in RFC 3339 format"),
    ]
    event_type: typing_extensions.Annotated[
        RouteEventResponseResponseBodyEventType,
        FieldMetadata(alias="eventType"),
        pydantic.Field(
            alias="eventType",
            description="Type of the event that occurred.  Valid values: `stopArrived`, `stopCompleted`, `stopEnRoute`, `stopSkipped`, `stopTaskCompleted`, `stopTaskSkipped`, `stopEtaUpdated`, `unspecified`",
        ),
    ]
    happened_at_time: typing_extensions.Annotated[
        dt.datetime,
        FieldMetadata(alias="happenedAtTime"),
        pydantic.Field(alias="happenedAtTime", description="Time the event happened in RFC 3339 format"),
    ]
    id: str = pydantic.Field()
    """
    Unique ID of the route event.
    """

    route: RouteEventRouteResponseResponseBody
    stop: typing.Optional[RouteEventStopResponseResponseBody] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

IssuesPatchIssueRequestBodyStatus = typing.Union[
    typing.Literal["open", "inProgress", "resolved", "dismissed"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .location_object_response_body import LocationObjectResponseBody


class RouteStopEstimatedArrivalDetailsObjectResponseBody(UniversalBaseModel):
    """
    Details specific to Route Stop Estimated Arrival
    """

    alert_before_arrival_milliseconds: typing_extensions.Annotated[
        int,
        FieldMetadata(alias="alertBeforeArrivalMilliseconds"),
        pydantic.Field(
            alias="alertBeforeArrivalMilliseconds",
            description="Time threshold for when to send an alert. Sends an alert when the ETA is less than the threshold.",
        ),
    ]
    has_live_share_link: typing_extensions.Annotated[
        typing.Optional[bool],
        FieldMetadata(alias="hasLiveShareLink"),
        pydantic.Field(
            alias="hasLiveShareLink",
            description="If true, will include a live sharing link in the alert. Defaults to false.",
        ),
    ] = None
    is_alert_on_route_stop_only: typing_extensions.Annotated[
        typing.Optional[bool],
        FieldMetadata(alias="isAlertOnRouteStopOnly"),
        pydantic.Field(
            alias="isAlertOnRouteStopOnly",
            description="If true, will only alert if the vehicle is en-route to the stop. Defaults to false.",
        ),
    ] = None
    location: LocationObjectResponseBody

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .function_code_package_response_body_status import FunctionCodePackageResponseBodyStatus


class FunctionCodePackageResponseBody(UniversalBaseModel):
    """
    Code package status for the Function.
    """

    download_get_url: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="downloadGetUrl"),
        pydantic.Field(
            alias="downloadGetUrl",
            description="A presigned GET URL for downloading the function's current code package. Present only when code has been uploaded.",
        ),
    ] = None
    status: FunctionCodePackageResponseBodyStatus = pydantic.Field()
    """
    Status of the code package.  Valid values: `unknown`, `pendingUpload`, `uploaded`, `deployed`
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

CarbCtcVehicleObjectResponseBodyTestStatus = typing.Union[
    typing.Literal["notScheduled", "scheduled", "inProgress", "awaitingResult", "pass", "fail", "error", "unknown"],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .voice_coaching_settings_object_response_body_events_to_coach_item import (
    VoiceCoachingSettingsObjectResponseBodyEventsToCoachItem,
)
from .voice_coaching_settings_object_response_body_language import VoiceCoachingSettingsObjectResponseBodyLanguage


class VoiceCoachingSettingsObjectResponseBody(UniversalBaseModel):
    """
    Enabling voice coaching will play messages for harsh events, speeding, and unbuckled seat belts.
    """

    events_to_coach: typing_extensions.Annotated[
        typing.Optional[typing.List[VoiceCoachingSettingsObjectResponseBodyEventsToCoachItem]],
        FieldMetadata(alias="eventsToCoach"),
        pydantic.Field(
            alias="eventsToCoach",
            description="Selected driving events will be enabled for voice coaching. Harsh driving events include harsh acceleration and harsh brake.",
        ),
    ] = None
    is_enabled: typing_extensions.Annotated[
        typing.Optional[bool],
        FieldMetadata(alias="isEnabled"),
        pydantic.Field(alias="isEnabled", description="Indicates whether voice coaching is turned on."),
    ] = None
    language: typing.Optional[VoiceCoachingSettingsObjectResponseBodyLanguage] = pydantic.Field(default=None)
    """
    The coaching language. The default language is English.  Valid values: `english`, `spanish`, `dutch`, `englishUk`, `italian`, `french`, `german`
    """

    speeding_threshold_mph: typing_extensions.Annotated[
        typing.Optional[float],
        FieldMetadata(alias="speedingThresholdMph"),
        pydantic.Field(alias="speedingThresholdMph", description="Alert when speed is over this many miles per hour."),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .dvir_author_signature import DvirAuthorSignature
from .dvir_license_plate import DvirLicensePlate
from .dvir_location import DvirLocation
from .dvir_mechanic_notes import DvirMechanicNotes
from .dvir_odometer_meters import DvirOdometerMeters
from .dvir_safety_status import DvirSafetyStatus
from .dvir_second_signature import DvirSecondSignature
from .dvir_third_signature import DvirThirdSignature
from .dvir_trailer import DvirTrailer
from .dvir_trailer_defects import DvirTrailerDefects
from .dvir_type import DvirType
from .dvir_vehicle import DvirVehicle
from .dvir_vehicle_defects import DvirVehicleDefects


class Dvir(UniversalBaseModel):
    """
    Information about a DVIR.
    """

    author_signature: typing_extensions.Annotated[
        typing.Optional[DvirAuthorSignature],
        FieldMetadata(alias="authorSignature"),
        pydantic.Field(alias="authorSignature"),
    ] = None
    end_time: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="endTime"),
        pydantic.Field(
            alias="endTime",
            description="Time when driver signed and completed this DVIR. UTC timestamp in RFC 3339 format. Example: `2020-01-27T07:06:25Z`.",
        ),
    ] = None
    id: str = pydantic.Field()
    """
    Unique Samsara ID for the DVIR.
    """

    license_plate: typing_extensions.Annotated[
        typing.Optional[DvirLicensePlate], FieldMetadata(alias="licensePlate"), pydantic.Field(alias="licensePlate")
    ] = None
    location: typing.Optional[DvirLocation] = None
    mechanic_notes: typing_extensions.Annotated[
        typing.Optional[DvirMechanicNotes], FieldMetadata(alias="mechanicNotes"), pydantic.Field(alias="mechanicNotes")
    ] = None
    odometer_meters: typing_extensions.Annotated[
        typing.Optional[DvirOdometerMeters],
        FieldMetadata(alias="odometerMeters"),
        pydantic.Field(alias="odometerMeters"),
    ] = None
    safety_status: typing_extensions.Annotated[
        typing.Optional[DvirSafetyStatus],
        FieldMetadata(alias="safetyStatus"),
        pydantic.Field(
            alias="safetyStatus",
            description="The condition of vehicle on which DVIR was done. Valid values: `safe`, `unsafe`, `resolved`.",
        ),
    ] = None
    second_signature: typing_extensions.Annotated[
        typing.Optional[DvirSecondSignature],
        FieldMetadata(alias="secondSignature"),
        pydantic.Field(alias="secondSignature"),
    ] = None
    start_time: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="startTime"),
        pydantic.Field(
            alias="startTime",
            description="Time when driver began filling out this DVIR. UTC timestamp in RFC 3339 format. Example: `2020-01-27T07:06:25Z`.",
        ),
    ] = None
    third_signature: typing_extensions.Annotated[
        typing.Optional[DvirThirdSignature],
        FieldMetadata(alias="thirdSignature"),
        pydantic.Field(alias="thirdSignature"),
    ] = None
    trailer: typing.Optional[DvirTrailer] = None
    trailer_defects: typing_extensions.Annotated[
        typing.Optional[DvirTrailerDefects],
        FieldMetadata(alias="trailerDefects"),
        pydantic.Field(alias="trailerDefects"),
    ] = None
    trailer_name: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="trailerName"),
        pydantic.Field(
            alias="trailerName",
            description="The name of the trailer the DVIR was submitted for.  Only included for tractor+trailer DVIRs.",
        ),
    ] = None
    type: typing.Optional[DvirType] = pydantic.Field(default=None)
    """
    Inspection type of the DVIR. Valid values: `preTrip`, `postTrip`, `mechanic`, `unspecified`.
    """

    vehicle: typing.Optional[DvirVehicle] = None
    vehicle_defects: typing_extensions.Annotated[
        typing.Optional[DvirVehicleDefects],
        FieldMetadata(alias="vehicleDefects"),
        pydantic.Field(alias="vehicleDefects"),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .vehicle_external_ids import VehicleExternalIds
from .vehicle_id import VehicleId
from .vehicle_name import VehicleName
from .vehicle_stats_ambient_air_temp_milli_c import VehicleStatsAmbientAirTempMilliC
from .vehicle_stats_aux_input import VehicleStatsAuxInput
from .vehicle_stats_barometric_pressure_pa import VehicleStatsBarometricPressurePa
from .vehicle_stats_battery_voltage import VehicleStatsBatteryVoltage
from .vehicle_stats_def_level_milli_percent import VehicleStatsDefLevelMilliPercent
from .vehicle_stats_ecu_speed_mph import VehicleStatsEcuSpeedMph
from .vehicle_stats_engine_coolant_temp_milli_c import VehicleStatsEngineCoolantTempMilliC
from .vehicle_stats_engine_immobilizer import VehicleStatsEngineImmobilizer
from .vehicle_stats_engine_load_percent import VehicleStatsEngineLoadPercent
from .vehicle_stats_engine_oil_pressure_k_pa import VehicleStatsEngineOilPressureKPa
from .vehicle_stats_engine_rpm import VehicleStatsEngineRpm
from .vehicle_stats_engine_state import VehicleStatsEngineState
from .vehicle_stats_fault_codes import VehicleStatsFaultCodes
from .vehicle_stats_fuel_consumed_milliliters import VehicleStatsFuelConsumedMilliliters
from .vehicle_stats_fuel_percent import VehicleStatsFuelPercent
from .vehicle_stats_gps import VehicleStatsGps
from .vehicle_stats_gps_distance_meters import VehicleStatsGpsDistanceMeters
from .vehicle_stats_gps_odometer_meters import VehicleStatsGpsOdometerMeters
from .vehicle_stats_idling_duration_milliseconds import VehicleStatsIdlingDurationMilliseconds
from .vehicle_stats_intake_manifold_temp_milli_c import VehicleStatsIntakeManifoldTempMilliC
from .vehicle_stats_nfc_card_scan import VehicleStatsNfcCardScan
from .vehicle_stats_obd_engine_seconds import VehicleStatsObdEngineSeconds
from .vehicle_stats_obd_odometer_meters import VehicleStatsObdOdometerMeters
from .vehicle_stats_response_ecu_door_status import VehicleStatsResponseEcuDoorStatus
from .vehicle_stats_response_ev_average_battery_temperature_milli_celsius import (
    VehicleStatsResponseEvAverageBatteryTemperatureMilliCelsius,
)
from .vehicle_stats_response_ev_battery_current_milli_amp import VehicleStatsResponseEvBatteryCurrentMilliAmp
from .vehicle_stats_response_ev_battery_state_of_health_milli_percent import (
    VehicleStatsResponseEvBatteryStateOfHealthMilliPercent,
)
from .vehicle_stats_response_ev_battery_voltage_milli_volt import VehicleStatsResponseEvBatteryVoltageMilliVolt
from .vehicle_stats_response_ev_charging_current_milli_amp import VehicleStatsResponseEvChargingCurrentMilliAmp
from .vehicle_stats_response_ev_charging_energy_micro_wh import VehicleStatsResponseEvChargingEnergyMicroWh
from .vehicle_stats_response_ev_charging_status import VehicleStatsResponseEvChargingStatus
from .vehicle_stats_response_ev_charging_voltage_milli_volt import VehicleStatsResponseEvChargingVoltageMilliVolt
from .vehicle_stats_response_ev_consumed_energy_micro_wh import VehicleStatsResponseEvConsumedEnergyMicroWh
from .vehicle_stats_response_ev_distance_driven_meters import VehicleStatsResponseEvDistanceDrivenMeters
from .vehicle_stats_response_ev_regenerated_energy_micro_wh import VehicleStatsResponseEvRegeneratedEnergyMicroWh
from .vehicle_stats_response_ev_state_of_charge_milli_percent import VehicleStatsResponseEvStateOfChargeMilliPercent
from .vehicle_stats_response_seatbelt_driver import VehicleStatsResponseSeatbeltDriver
from .vehicle_stats_response_spreader_active import VehicleStatsResponseSpreaderActive
from .vehicle_stats_response_spreader_air_temp import VehicleStatsResponseSpreaderAirTemp
from .vehicle_stats_response_spreader_blast_state import VehicleStatsResponseSpreaderBlastState
from .vehicle_stats_response_spreader_granular_name import VehicleStatsResponseSpreaderGranularName
from .vehicle_stats_response_spreader_granular_rate import VehicleStatsResponseSpreaderGranularRate
from .vehicle_stats_response_spreader_liquid_name import VehicleStatsResponseSpreaderLiquidName
from .vehicle_stats_response_spreader_liquid_rate import VehicleStatsResponseSpreaderLiquidRate
from .vehicle_stats_response_spreader_on_state import VehicleStatsResponseSpreaderOnState
from .vehicle_stats_response_spreader_plow_status import VehicleStatsResponseSpreaderPlowStatus
from .vehicle_stats_response_spreader_prewet_name import VehicleStatsResponseSpreaderPrewetName
from .vehicle_stats_response_spreader_prewet_rate import VehicleStatsResponseSpreaderPrewetRate
from .vehicle_stats_response_spreader_road_temp import VehicleStatsResponseSpreaderRoadTemp
from .vehicle_stats_synthetic_engine_seconds import VehicleStatsSyntheticEngineSeconds


class VehicleStatsResponseData(UniversalBaseModel):
    """
    A vehicle and its most recent stat.
    """

    ambient_air_temperature_milli_c: typing_extensions.Annotated[
        typing.Optional[VehicleStatsAmbientAirTempMilliC],
        FieldMetadata(alias="ambientAirTemperatureMilliC"),
        pydantic.Field(alias="ambientAirTemperatureMilliC"),
    ] = None
    aux_input_1: typing_extensions.Annotated[
        typing.Optional[VehicleStatsAuxInput], FieldMetadata(alias="auxInput1"), pydantic.Field(alias="auxInput1")
    ] = None
    aux_input_10: typing_extensions.Annotated[
        typing.Optional[VehicleStatsAuxInput], FieldMetadata(alias="auxInput10"), pydantic.Field(alias="auxInput10")
    ] = None
    aux_input_11: typing_extensions.Annotated[
        typing.Optional[VehicleStatsAuxInput], FieldMetadata(alias="auxInput11"), pydantic.Field(alias="auxInput11")
    ] = None
    aux_input_12: typing_extensions.Annotated[
        typing.Optional[VehicleStatsAuxInput], FieldMetadata(alias="auxInput12"), pydantic.Field(alias="auxInput12")
    ] = None
    aux_input_13: typing_extensions.Annotated[
        typing.Optional[VehicleStatsAuxInput], FieldMetadata(alias="auxInput13"), pydantic.Field(alias="auxInput13")
    ] = None
    aux_input_2: typing_extensions.Annotated[
        typing.Optional[VehicleStatsAuxInput], FieldMetadata(alias="auxInput2"), pydantic.Field(alias="auxInput2")
    ] = None
    aux_input_3: typing_extensions.Annotated[
        typing.Optional[VehicleStatsAuxInput], FieldMetadata(alias="auxInput3"), pydantic.Field(alias="auxInput3")
    ] = None
    aux_input_4: typing_extensions.Annotated[
        typing.Optional[VehicleStatsAuxInput], FieldMetadata(alias="auxInput4"), pydantic.Field(alias="auxInput4")
    ] = None
    aux_input_5: typing_extensions.Annotated[
        typing.Optional[VehicleStatsAuxInput], FieldMetadata(alias="auxInput5"), pydantic.Field(alias="auxInput5")
    ] = None
    aux_input_6: typing_extensions.Annotated[
        typing.Optional[VehicleStatsAuxInput], FieldMetadata(alias="auxInput6"), pydantic.Field(alias="auxInput6")
    ] = None
    aux_input_7: typing_extensions.Annotated[
        typing.Optional[VehicleStatsAuxInput], FieldMetadata(alias="auxInput7"), pydantic.Field(alias="auxInput7")
    ] = None
    aux_input_8: typing_extensions.Annotated[
        typing.Optional[VehicleStatsAuxInput], FieldMetadata(alias="auxInput8"), pydantic.Field(alias="auxInput8")
    ] = None
    aux_input_9: typing_extensions.Annotated[
        typing.Optional[VehicleStatsAuxInput], FieldMetadata(alias="auxInput9"), pydantic.Field(alias="auxInput9")
    ] = None
    barometric_pressure_pa: typing_extensions.Annotated[
        typing.Optional[VehicleStatsBarometricPressurePa],
        FieldMetadata(alias="barometricPressurePa"),
        pydantic.Field(alias="barometricPressurePa"),
    ] = None
    battery_milli_volts: typing_extensions.Annotated[
        typing.Optional[VehicleStatsBatteryVoltage],
        FieldMetadata(alias="batteryMilliVolts"),
        pydantic.Field(alias="batteryMilliVolts"),
    ] = None
    def_level_milli_percent: typing_extensions.Annotated[
        typing.Optional[VehicleStatsDefLevelMilliPercent],
        FieldMetadata(alias="defLevelMilliPercent"),
        pydantic.Field(alias="defLevelMilliPercent"),
    ] = None
    ecu_door_status: typing_extensions.Annotated[
        typing.Optional[VehicleStatsResponseEcuDoorStatus],
        FieldMetadata(alias="ecuDoorStatus"),
        pydantic.Field(alias="ecuDoorStatus"),
    ] = None
    ecu_speed_mph: typing_extensions.Annotated[
        typing.Optional[VehicleStatsEcuSpeedMph],
        FieldMetadata(alias="ecuSpeedMph"),
        pydantic.Field(alias="ecuSpeedMph"),
    ] = None
    engine_coolant_temperature_milli_c: typing_extensions.Annotated[
        typing.Optional[VehicleStatsEngineCoolantTempMilliC],
        FieldMetadata(alias="engineCoolantTemperatureMilliC"),
        pydantic.Field(alias="engineCoolantTemperatureMilliC"),
    ] = None
    engine_immobilizer: typing_extensions.Annotated[
        typing.Optional[VehicleStatsEngineImmobilizer],
        FieldMetadata(alias="engineImmobilizer"),
        pydantic.Field(alias="engineImmobilizer"),
    ] = None
    engine_load_percent: typing_extensions.Annotated[
        typing.Optional[VehicleStatsEngineLoadPercent],
        FieldMetadata(alias="engineLoadPercent"),
        pydantic.Field(alias="engineLoadPercent"),
    ] = None
    engine_oil_pressure_k_pa: typing_extensions.Annotated[
        typing.Optional[VehicleStatsEngineOilPressureKPa],
        FieldMetadata(alias="engineOilPressureKPa"),
        pydantic.Field(alias="engineOilPressureKPa"),
    ] = None
    engine_rpm: typing_extensions.Annotated[
        typing.Optional[VehicleStatsEngineRpm], FieldMetadata(alias="engineRpm"), pydantic.Field(alias="engineRpm")
    ] = None
    engine_state: typing_extensions.Annotated[
        typing.Optional[VehicleStatsEngineState],
        FieldMetadata(alias="engineState"),
        pydantic.Field(alias="engineState"),
    ] = None
    ev_average_battery_temperature_milli_celsius: typing_extensions.Annotated[
        typing.Optional[VehicleStatsResponseEvAverageBatteryTemperatureMilliCelsius],
        FieldMetadata(alias="evAverageBatteryTemperatureMilliCelsius"),
        pydantic.Field(alias="evAverageBatteryTemperatureMilliCelsius"),
    ] = None
    ev_battery_current_milli_amp: typing_extensions.Annotated[
        typing.Optional[VehicleStatsResponseEvBatteryCurrentMilliAmp],
        FieldMetadata(alias="evBatteryCurrentMilliAmp"),
        pydantic.Field(alias="evBatteryCurrentMilliAmp"),
    ] = None
    ev_battery_state_of_health_milli_percent: typing_extensions.Annotated[
        typing.Optional[VehicleStatsResponseEvBatteryStateOfHealthMilliPercent],
        FieldMetadata(alias="evBatteryStateOfHealthMilliPercent"),
        pydantic.Field(alias="evBatteryStateOfHealthMilliPercent"),
    ] = None
    ev_battery_voltage_milli_volt: typing_extensions.Annotated[
        typing.Optional[VehicleStatsResponseEvBatteryVoltageMilliVolt],
        FieldMetadata(alias="evBatteryVoltageMilliVolt"),
        pydantic.Field(alias="evBatteryVoltageMilliVolt"),
    ] = None
    ev_charging_current_milli_amp: typing_extensions.Annotated[
        typing.Optional[VehicleStatsResponseEvChargingCurrentMilliAmp],
        FieldMetadata(alias="evChargingCurrentMilliAmp"),
        pydantic.Field(alias="evChargingCurrentMilliAmp"),
    ] = None
    ev_charging_energy_micro_wh: typing_extensions.Annotated[
        typing.Optional[VehicleStatsResponseEvChargingEnergyMicroWh],
        FieldMetadata(alias="evChargingEnergyMicroWh"),
        pydantic.Field(alias="evChargingEnergyMicroWh"),
    ] = None
    ev_charging_status: typing_extensions.Annotated[
        typing.Optional[VehicleStatsResponseEvChargingStatus],
        FieldMetadata(alias="evChargingStatus"),
        pydantic.Field(alias="evChargingStatus"),
    ] = None
    ev_charging_voltage_milli_volt: typing_extensions.Annotated[
        typing.Optional[VehicleStatsResponseEvChargingVoltageMilliVolt],
        FieldMetadata(alias="evChargingVoltageMilliVolt"),
        pydantic.Field(alias="evChargingVoltageMilliVolt"),
    ] = None
    ev_consumed_energy_micro_wh: typing_extensions.Annotated[
        typing.Optional[VehicleStatsResponseEvConsumedEnergyMicroWh],
        FieldMetadata(alias="evConsumedEnergyMicroWh"),
        pydantic.Field(alias="evConsumedEnergyMicroWh"),
    ] = None
    ev_distance_driven_meters: typing_extensions.Annotated[
        typing.Optional[VehicleStatsResponseEvDistanceDrivenMeters],
        FieldMetadata(alias="evDistanceDrivenMeters"),
        pydantic.Field(alias="evDistanceDrivenMeters"),
    ] = None
    ev_regenerated_energy_micro_wh: typing_extensions.Annotated[
        typing.Optional[VehicleStatsResponseEvRegeneratedEnergyMicroWh],
        FieldMetadata(alias="evRegeneratedEnergyMicroWh"),
        pydantic.Field(alias="evRegeneratedEnergyMicroWh"),
    ] = None
    ev_state_of_charge_milli_percent: typing_extensions.Annotated[
        typing.Optional[VehicleStatsResponseEvStateOfChargeMilliPercent],
        FieldMetadata(alias="evStateOfChargeMilliPercent"),
        pydantic.Field(alias="evStateOfChargeMilliPercent"),
    ] = None
    external_ids: typing_extensions.Annotated[
        typing.Optional[VehicleExternalIds], FieldMetadata(alias="externalIds"), pydantic.Field(alias="externalIds")
    ] = None
    fault_codes: typing_extensions.Annotated[
        typing.Optional[VehicleStatsFaultCodes], FieldMetadata(alias="faultCodes"), pydantic.Field(alias="faultCodes")
    ] = None
    fuel_consumed_milliliters: typing_extensions.Annotated[
        typing.Optional[VehicleStatsFuelConsumedMilliliters],
        FieldMetadata(alias="fuelConsumedMilliliters"),
        pydantic.Field(alias="fuelConsumedMilliliters"),
    ] = None
    fuel_percent: typing_extensions.Annotated[
        typing.Optional[VehicleStatsFuelPercent],
        FieldMetadata(alias="fuelPercent"),
        pydantic.Field(alias="fuelPercent"),
    ] = None
    gps: typing.Optional[VehicleStatsGps] = None
    gps_distance_meters: typing_extensions.Annotated[
        typing.Optional[VehicleStatsGpsDistanceMeters],
        FieldMetadata(alias="gpsDistanceMeters"),
        pydantic.Field(alias="gpsDistanceMeters"),
    ] = None
    gps_odometer_meters: typing_extensions.Annotated[
        typing.Optional[VehicleStatsGpsOdometerMeters],
        FieldMetadata(alias="gpsOdometerMeters"),
        pydantic.Field(alias="gpsOdometerMeters"),
    ] = None
    id: VehicleId
    idling_duration_milliseconds: typing_extensions.Annotated[
        typing.Optional[VehicleStatsIdlingDurationMilliseconds],
        FieldMetadata(alias="idlingDurationMilliseconds"),
        pydantic.Field(alias="idlingDurationMilliseconds"),
    ] = None
    intake_manifold_temperature_milli_c: typing_extensions.Annotated[
        typing.Optional[VehicleStatsIntakeManifoldTempMilliC],
        FieldMetadata(alias="intakeManifoldTemperatureMilliC"),
        pydantic.Field(alias="intakeManifoldTemperatureMilliC"),
    ] = None
    name: VehicleName
    nfc_card_scan: typing_extensions.Annotated[
        typing.Optional[VehicleStatsNfcCardScan],
        FieldMetadata(alias="nfcCardScan"),
        pydantic.Field(alias="nfcCardScan"),
    ] = None
    obd_engine_seconds: typing_extensions.Annotated[
        typing.Optional[VehicleStatsObdEngineSeconds],
        FieldMetadata(alias="obdEngineSeconds"),
        pydantic.Field(alias="obdEngineSeconds"),
    ] = None
    obd_odometer_meters: typing_extensions.Annotated[
        typing.Optional[VehicleStatsObdOdometerMeters],
        FieldMetadata(alias="obdOdometerMeters"),
        pydantic.Field(alias="obdOdometerMeters"),
    ] = None
    seatbelt_driver: typing_extensions.Annotated[
        typing.Optional[VehicleStatsResponseSeatbeltDriver],
        FieldMetadata(alias="seatbeltDriver"),
        pydantic.Field(alias="seatbeltDriver"),
    ] = None
    spreader_active: typing_extensions.Annotated[
        typing.Optional[VehicleStatsResponseSpreaderActive],
        FieldMetadata(alias="spreaderActive"),
        pydantic.Field(alias="spreaderActive"),
    ] = None
    spreader_air_temp: typing_extensions.Annotated[
        typing.Optional[VehicleStatsResponseSpreaderAirTemp],
        FieldMetadata(alias="spreaderAirTemp"),
        pydantic.Field(alias="spreaderAirTemp"),
    ] = None
    spreader_blast_state: typing_extensions.Annotated[
        typing.Optional[VehicleStatsResponseSpreaderBlastState],
        FieldMetadata(alias="spreaderBlastState"),
        pydantic.Field(alias="spreaderBlastState"),
    ] = None
    spreader_granular_name: typing_extensions.Annotated[
        typing.Optional[VehicleStatsResponseSpreaderGranularName],
        FieldMetadata(alias="spreaderGranularName"),
        pydantic.Field(alias="spreaderGranularName"),
    ] = None
    spreader_granular_rate: typing_extensions.Annotated[
        typing.Optional[VehicleStatsResponseSpreaderGranularRate],
        FieldMetadata(alias="spreaderGranularRate"),
        pydantic.Field(alias="spreaderGranularRate"),
    ] = None
    spreader_liquid_name: typing_extensions.Annotated[
        typing.Optional[VehicleStatsResponseSpreaderLiquidName],
        FieldMetadata(alias="spreaderLiquidName"),
        pydantic.Field(alias="spreaderLiquidName"),
    ] = None
    spreader_liquid_rate: typing_extensions.Annotated[
        typing.Optional[VehicleStatsResponseSpreaderLiquidRate],
        FieldMetadata(alias="spreaderLiquidRate"),
        pydantic.Field(alias="spreaderLiquidRate"),
    ] = None
    spreader_on_state: typing_extensions.Annotated[
        typing.Optional[VehicleStatsResponseSpreaderOnState],
        FieldMetadata(alias="spreaderOnState"),
        pydantic.Field(alias="spreaderOnState"),
    ] = None
    spreader_plow_status: typing_extensions.Annotated[
        typing.Optional[VehicleStatsResponseSpreaderPlowStatus],
        FieldMetadata(alias="spreaderPlowStatus"),
        pydantic.Field(alias="spreaderPlowStatus"),
    ] = None
    spreader_prewet_name: typing_extensions.Annotated[
        typing.Optional[VehicleStatsResponseSpreaderPrewetName],
        FieldMetadata(alias="spreaderPrewetName"),
        pydantic.Field(alias="spreaderPrewetName"),
    ] = None
    spreader_prewet_rate: typing_extensions.Annotated[
        typing.Optional[VehicleStatsResponseSpreaderPrewetRate],
        FieldMetadata(alias="spreaderPrewetRate"),
        pydantic.Field(alias="spreaderPrewetRate"),
    ] = None
    spreader_road_temp: typing_extensions.Annotated[
        typing.Optional[VehicleStatsResponseSpreaderRoadTemp],
        FieldMetadata(alias="spreaderRoadTemp"),
        pydantic.Field(alias="spreaderRoadTemp"),
    ] = None
    synthetic_engine_seconds: typing_extensions.Annotated[
        typing.Optional[VehicleStatsSyntheticEngineSeconds],
        FieldMetadata(alias="syntheticEngineSeconds"),
        pydantic.Field(alias="syntheticEngineSeconds"),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .goa_gateway_tiny_response_response_body_model import GoaGatewayTinyResponseResponseBodyModel


class GoaGatewayTinyResponseResponseBody(UniversalBaseModel):
    """
    A minified gateway object
    """

    model: GoaGatewayTinyResponseResponseBodyModel = pydantic.Field()
    """
    The model of the gateway installed on the asset.  Valid values: `AG15`, `AG24`, `AG24EU`, `AG26`, `AG26EU`, `AG41`, `AG41EU`, `AG45`, `AG45EU`, `AG46`, `AG46EU`, `AG46P`, `AG46PEU`, `AG51`, `AG51EU`, `AG52`, `AG52EU`, `AG53`, `AG53EU`, `IG15`, `IG21`, `IG41`, `IG61`, `SG1`, `SG1B`, `SG1G`, `SG1G32`, `SG1x`, `VG32`, `VG33`, `VG34`, `VG34EU`, `VG34FN`, `VG34M`, `VG54ATT`, `VG54EU`, `VG54FN`, `VG54NA`, `VG54NAE`, `VG54NAH`, `VG55EU`, `VG55FN`, `VG55NA`
    """

    serial: str = pydantic.Field()
    """
    The serial number of the gateway installed on the asset.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .patch_safety_events_response_item_response_body import PatchSafetyEventsResponseItemResponseBody


class SafetyEventsV2PatchSafetyEventsV2BatchResponseBody(UniversalBaseModel):
    request_id: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="requestId"),
        pydantic.Field(
            alias="requestId",
            description="Server-generated UUID for client-side correlation and logging. Not stored or queryable on the backend.",
        ),
    ]
    responses: typing.List[PatchSafetyEventsResponseItemResponseBody] = pydantic.Field()
    """
    One entry per requested safetyEventId, in the same order as the request.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .alert_object_product_response_body import AlertObjectProductResponseBody
from .goa_tag_tiny_response_response_body import GoaTagTinyResponseResponseBody


class AlertObjectSensorResponseBody(UniversalBaseModel):
    """
    A sensor associated with the alert.
    """

    id: str = pydantic.Field()
    """
    Thye ID of the sensor associated with the alert
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The name of the sensor.
    """

    pinned_device_id: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="pinnedDeviceId"),
        pydantic.Field(alias="pinnedDeviceId", description="The Pinned Device ID associated with the alert"),
    ] = None
    product: typing.Optional[AlertObjectProductResponseBody] = None
    tags: typing.Optional[typing.List[GoaTagTinyResponseResponseBody]] = pydantic.Field(default=None)
    """
    The list of [tags](https://kb.samsara.com/hc/en-us/articles/360026674631-Using-Tags-and-Tag-Nesting) associated with the sensor.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.driver_qr_codes_create_driver_qr_code_response_body import DriverQrCodesCreateDriverQrCodeResponseBody
from ..types.driver_qr_codes_get_drivers_qr_codes_response_body import DriverQrCodesGetDriversQrCodesResponseBody
from .raw_client import AsyncRawDriverQrCodesClient, RawDriverQrCodesClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class DriverQrCodesClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawDriverQrCodesClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawDriverQrCodesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawDriverQrCodesClient
        """
        return self._raw_client

    def get_drivers_qr_codes(
        self,
        *,
        driver_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> DriverQrCodesGetDriversQrCodesResponseBody:
        """
        Get details for requested driver(s) QR code, used for driver trip assignment.

         <b>Rate limit:</b> 5 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Read Drivers** under the Drivers category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        driver_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            String of comma separated driver IDs. List of driver - QR codes for specified driver(s) will be returned.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DriverQrCodesGetDriversQrCodesResponseBody
            OK response.

        Examples
        --------
        from samsara import Samsara

        client = Samsara(
            token="YOUR_TOKEN",
        )
        client.driver_qr_codes.get_drivers_qr_codes()
        """
        _response = self._raw_client.get_drivers_qr_codes(driver_ids=driver_ids, request_options=request_options)
        return _response.data

    def create_driver_qr_code(
        self, *, driver_id: int, request_options: typing.Optional[RequestOptions] = None
    ) -> DriverQrCodesCreateDriverQrCodeResponseBody:
        """
        Assign a new QR code for the requested driver. Return error if an active QR code already exists.

         <b>Rate limit:</b> 100 requests/min (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Write Drivers** under the Drivers category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        driver_id : int
            Unique ID of the driver.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DriverQrCodesCreateDriverQrCodeResponseBody
            OK response.

        Examples
        --------
        from samsara import Samsara

        client = Samsara(
            token="YOUR_TOKEN",
        )
        client.driver_qr_codes.create_driver_qr_code(
            driver_id=494123,
        )
        """
        _response = self._raw_client.create_driver_qr_code(driver_id=driver_id, request_options=request_options)
        return _response.data

    def delete_driver_qr_code(self, *, driver_id: int, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Revoke requested driver's currently active QR code.

         <b>Rate limit:</b> 100 requests/min (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Write Drivers** under the Drivers category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        driver_id : int
            Unique ID of the driver.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from samsara import Samsara

        client = Samsara(
            token="YOUR_TOKEN",
        )
        client.driver_qr_codes.delete_driver_qr_code(
            driver_id=494123,
        )
        """
        _response = self._raw_client.delete_driver_qr_code(driver_id=driver_id, request_options=request_options)
        return _response.data


class AsyncDriverQrCodesClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawDriverQrCodesClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawDriverQrCodesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawDriverQrCodesClient
        """
        return self._raw_client

    async def get_drivers_qr_codes(
        self,
        *,
        driver_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> DriverQrCodesGetDriversQrCodesResponseBody:
        """
        Get details for requested driver(s) QR code, used for driver trip assignment.

         <b>Rate limit:</b> 5 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Read Drivers** under the Drivers category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        driver_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            String of comma separated driver IDs. List of driver - QR codes for specified driver(s) will be returned.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DriverQrCodesGetDriversQrCodesResponseBody
            OK response.

        Examples
        --------
        import asyncio

        from samsara import AsyncSamsara

        client = AsyncSamsara(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.driver_qr_codes.get_drivers_qr_codes()


        asyncio.run(main())
        """
        _response = await self._raw_client.get_drivers_qr_codes(driver_ids=driver_ids, request_options=request_options)
        return _response.data

    async def create_driver_qr_code(
        self, *, driver_id: int, request_options: typing.Optional[RequestOptions] = None
    ) -> DriverQrCodesCreateDriverQrCodeResponseBody:
        """
        Assign a new QR code for the requested driver. Return error if an active QR code already exists.

         <b>Rate limit:</b> 100 requests/min (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Write Drivers** under the Drivers category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        driver_id : int
            Unique ID of the driver.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DriverQrCodesCreateDriverQrCodeResponseBody
            OK response.

        Examples
        --------
        import asyncio

        from samsara import AsyncSamsara

        client = AsyncSamsara(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.driver_qr_codes.create_driver_qr_code(
                driver_id=494123,
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create_driver_qr_code(driver_id=driver_id, request_options=request_options)
        return _response.data

    async def delete_driver_qr_code(
        self, *, driver_id: int, request_options: typing.Optional[RequestOptions] = None
    ) -> None:
        """
        Revoke requested driver's currently active QR code.

         <b>Rate limit:</b> 100 requests/min (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Write Drivers** under the Drivers category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        driver_id : int
            Unique ID of the driver.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from samsara import AsyncSamsara

        client = AsyncSamsara(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.driver_qr_codes.delete_driver_qr_code(
                driver_id=494123,
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete_driver_qr_code(driver_id=driver_id, request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .function_code_package_response_body import FunctionCodePackageResponseBody
from .function_config_response_body import FunctionConfigResponseBody
from .function_effects_response_body import FunctionEffectsResponseBody


class CreateFunctionDetailResponseBody(UniversalBaseModel):
    """
    Details of a newly created Function, including a URL for uploading the code package. After uploading, call the deploy endpoint to make the function runnable.
    """

    code_package: typing_extensions.Annotated[
        FunctionCodePackageResponseBody, FieldMetadata(alias="codePackage"), pydantic.Field(alias="codePackage")
    ]
    config: FunctionConfigResponseBody
    created_at_time: typing_extensions.Annotated[
        dt.datetime,
        FieldMetadata(alias="createdAtTime"),
        pydantic.Field(alias="createdAtTime", description="RFC 3339 timestamp when the Function was created."),
    ]
    description: typing.Optional[str] = pydantic.Field(default=None)
    """
    Description of the Function.
    """

    effects: FunctionEffectsResponseBody
    last_update_timestamp_ms: typing_extensions.Annotated[
        int,
        FieldMetadata(alias="lastUpdateTimestampMs"),
        pydantic.Field(
            alias="lastUpdateTimestampMs",
            description="Epoch milliseconds of the last update. Use this value for optimistic locking in PATCH requests.",
        ),
    ]
    name: str = pydantic.Field()
    """
    Name of the Function.
    """

    updated_at_time: typing_extensions.Annotated[
        dt.datetime,
        FieldMetadata(alias="updatedAtTime"),
        pydantic.Field(alias="updatedAtTime", description="RFC 3339 timestamp when the Function was last updated."),
    ]
    upload_put_url: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="uploadPutUrl"),
        pydantic.Field(
            alias="uploadPutUrl",
            description="A presigned PUT URL for uploading the function's code package. Valid for a limited time. After uploading, call `POST /functions/{name}/deploy` to make the function runnable.",
        ),
    ]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

VehicleResponseObjectResponseBodyVehicleType = typing.Union[
    typing.Literal["unset", "passenger", "truck", "bus"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .vertex_response_body import VertexResponseBody


class PolygonResponseBody(UniversalBaseModel):
    """
    Information about a polygon geofence. This field is only needed if the geofence is a polygon.
    """

    name: str = pydantic.Field()
    """
    The name of the polygon.
    """

    vertices: typing.Optional[typing.List[VertexResponseBody]] = pydantic.Field(default=None)
    """
    The vertices of the polygon geofence. These geofence vertices describe the perimeter of the polygon, and must consist of at least 3 vertices and less than 40.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
